﻿Imports System.Collections
Imports System.ComponentModel
Imports System.Collections.ObjectModel

Public Class Product
    Implements INotifyPropertyChanged
    Implements IEditableObject
    Shared _lines As String() = "Computers|Washers|Stoves".Split("|"c)
    Shared _colors As String() = "Red|Green|Blue|White".Split("|"c)
    ' object model
    Public Property Line() As String
        Get
            Return DirectCast(GetValue("Line"), String)
        End Get
        Set(value As String)
            SetValue("Line", value)
        End Set
    End Property
    Public Property Color() As String
        Get
            Return DirectCast(GetValue("Color"), String)
        End Get
        Set(value As String)
            SetValue("Color", value)
        End Set
    End Property
    Public Property Name() As String
        Get
            Return DirectCast(GetValue("Name"), String)
        End Get
        Set(value As String)
            SetValue("Name", value)
        End Set
    End Property
    Public Property Price() As Double
        Get
            Return CDbl(GetValue("Price"))
        End Get
        Set(value As Double)
            SetValue("Price", value)
        End Set
    End Property
    Public Property Weight() As Double
        Get
            Return CDbl(GetValue("Weight"))
        End Get
        Set(value As Double)
            SetValue("Weight", value)
        End Set
    End Property
    Public Property Cost() As Double
        Get
            Return CDbl(GetValue("Cost"))
        End Get
        Set(value As Double)
            SetValue("Cost", value)
        End Set
    End Property
    Public Property Volume() As Double
        Get
            Return CDbl(GetValue("Volume"))
        End Get
        Set(value As Double)
            SetValue("Volume", value)
        End Set
    End Property
    Public Property Discontinued() As Boolean
        Get
            Return CBool(GetValue("Discontinued"))
        End Get
        Set(value As Boolean)
            SetValue("Discontinued", value)
        End Set
    End Property
    Public Property Rating() As Integer
        Get
            Return CInt(GetValue("Rating"))
        End Get
        Set(value As Integer)
            SetValue("Rating", value)
        End Set
    End Property
    Public Property Special() As System.Nullable(Of Boolean)
        Get
            Return DirectCast(GetValue("Special"), System.Nullable(Of Boolean))
        End Get
        Set(value As System.Nullable(Of Boolean))
            SetValue("Special", value)
        End Set
    End Property
    Public Property Launched() As System.Nullable(Of DateTime)
        Get
            Return DirectCast(GetValue("Launched"), System.Nullable(Of DateTime))
        End Get
        Set(value As System.Nullable(Of DateTime))
            SetValue("Launched", value)
        End Set
    End Property

    ' get/set values
    Private _values As New Dictionary(Of String, Object)()
    Private Function GetValue(p As String) As Object
        Dim value As Object
        _values.TryGetValue(p, value)
        Return value
    End Function
    Private Sub SetValue(p As String, value As Object)
        If Not Object.Equals(value, GetValue(p)) Then
            _values(p) = value
            OnPropertyChanged(p)
        End If
    End Sub
    Protected Overridable Sub OnPropertyChanged(p As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(p))
    End Sub

    ' factory
    Public Shared Function GetProducts(count As Integer) As IList(Of Product)
        Dim list = New ObservableCollection(Of Product)()
        Dim rnd = New Random(0)
        For i As Integer = 0 To count - 1
            Dim p = New Product()
            p.Line = _lines(rnd.[Next]() Mod _lines.Length)
            p.Color = _colors(rnd.[Next]() Mod _colors.Length)
            If rnd.NextDouble() > 0.2 Then
                p.Name = String.Format("{0} {1}{2}", p.Line.Substring(0, p.Line.Length - 1), p.Line(0), i)
            End If
            p.Price = rnd.NextDouble() * 1000
            p.Weight = rnd.NextDouble() * 100
            p.Cost = rnd.NextDouble() * 600
            p.Volume = 500 + rnd.NextDouble() * 4000
            p.Discontinued = rnd.NextDouble() < 0.1
            p.Rating = rnd.[Next](0, 5)
            If rnd.NextDouble() > 0.2 Then
                p.Special = rnd.NextDouble() > 0.5
            End If
            If rnd.NextDouble() > 0.2 Then
                p.Launched = DateTime.Today.AddDays(rnd.[Next](-3000, -300))
            End If
            list.Add(p)
        Next
        Return list
    End Function
    Public Shared Function GetLines() As String()
        Return _lines
    End Function
    Public Shared Function GetColors() As String()
        Return _colors
    End Function

    Shared _clone As Dictionary(Of String, Object)
    Public Event PropertyChanged(sender As Object, e As System.ComponentModel.PropertyChangedEventArgs) Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged

    Public Sub BeginEdit() Implements System.ComponentModel.IEditableObject.BeginEdit
        If _clone Is Nothing Then
            _clone = New Dictionary(Of String, Object)()
        End If
        _clone.Clear()

        For Each key As String In _values.Keys
            _clone(CStr(key)) = _values(CStr(key))
        Next
    End Sub

    Public Sub CancelEdit() Implements System.ComponentModel.IEditableObject.CancelEdit
        _values.Clear()

        For Each key As String In _clone.Keys
            _values(CStr(key)) = _clone(CStr(key))
        Next
        OnPropertyChanged(Nothing)
    End Sub

    Public Sub EndEdit() Implements System.ComponentModel.IEditableObject.EndEdit

    End Sub
End Class
